@@ -36,6 +36,8 @@ enum {
 
 	/* Errors while handling job processes */
 	JOB_PROCESS_ERROR,
+	JOB_PROCESS_INVALID_SETUID,
+	JOB_PROCESS_INVALID_SETGID,
 
 	/* Errors while parsing configuration files */
 	PARSE_ILLEGAL_INTERVAL,
@@ -59,6 +61,8 @@ enum {
 #define ENVIRON_UNKNOWN_PARAM_STR	N_("Unknown parameter")
 #define ENVIRON_EXPECTED_OPERATOR_STR	N_("Expected operator")
 #define ENVIRON_MISMATCHED_BRACES_STR	N_("Mismatched braces")
+#define JOB_PROCESS_INVALID_SETUID_STR	N_("Invalid setuid user name does not exist")
+#define JOB_PROCESS_INVALID_SETGID_STR	N_("Invalid setgid group name does not exist")
 #define PARSE_ILLEGAL_INTERVAL_STR	N_("Illegal interval, expected number of seconds")
 #define PARSE_ILLEGAL_EXIT_STR		N_("Illegal exit status, expected integer")
 #define PARSE_ILLEGAL_SIGNAL_STR	N_("Illegal signal status, expected integer")
