@@ -584,29 +584,35 @@ job_process_spawn (Job          *job,
 		sigemptyset (&ignore.sa_mask);
 
 		if (sigaction (SIGCHLD, &ignore, &act) < 0) {
-			job_process_error_abort (fds[1], JOB_PROCESS_ERROR_OPENPT_MASTER, 0);
+			nih_error_raise_system ();
+			job_process_error_abort (fds[1], JOB_PROCESS_ERROR_SIGNAL, 0);
 		}
 
 		if (grantpt (pty_master) < 0) {
-			job_process_error_abort (fds[1], JOB_PROCESS_ERROR_OPENPT_MASTER, 0);
+			nih_error_raise_system ();
+			job_process_error_abort (fds[1], JOB_PROCESS_ERROR_GRANTPT, 0);
 		}
 
 		/* Restore child handler */
 		if (sigaction (SIGCHLD, &act, NULL) < 0) {
-			job_process_error_abort (fds[1], JOB_PROCESS_ERROR_OPENPT_MASTER, 0);
+			nih_error_raise_system ();
+			job_process_error_abort (fds[1], JOB_PROCESS_ERROR_SIGNAL, 0);
 		}
 
 		if (unlockpt (pty_master) < 0) {
+			nih_error_raise_system ();
 			job_process_error_abort (fds[1], JOB_PROCESS_ERROR_UNLOCKPT, 0);
 		}
 
 		if (ptsname_r (pty_master, pts_name, sizeof(pts_name)) < 0) {
+			nih_error_raise_system ();
 			job_process_error_abort (fds[1], JOB_PROCESS_ERROR_PTSNAME, 0);
 		}
 
 		pty_slave = open (pts_name, O_RDWR | O_NOCTTY);
 
 		if (pty_slave < 0) {
+			nih_error_raise_system ();
 			job_process_error_abort (fds[1], JOB_PROCESS_ERROR_OPENPT_SLAVE, 0);
 		}
 
@@ -618,8 +624,10 @@ job_process_spawn (Job          *job,
 	 */
 	if ((script_fd != -1) && (script_fd != JOB_PROCESS_SCRIPT_FD)) {
 		int tmp = dup2 (script_fd, JOB_PROCESS_SCRIPT_FD);
-		if (tmp < 0)
+		if (tmp < 0) {
+			nih_error_raise_system ();
 			job_process_error_abort (fds[1], JOB_PROCESS_ERROR_DUP, 0);
+		}
 		close (script_fd);
 		script_fd = tmp;
 	}
@@ -629,7 +637,7 @@ job_process_spawn (Job          *job,
 	 */
 	setsid ();
 
-	/* Set the process environment from the function paramters. */
+	/* Set the process environment from the function parameters. */
 	environ = (char **)env;
 
 	/* Handle unprivileged user job by dropping privileges to
@@ -658,22 +666,23 @@ job_process_spawn (Job          *job,
 			 * session in the chroot via D-Bus, so disallow
 			 * all jobs in such an environment.
 			 */
-			nih_return_error (-1, EPERM, "user jobs not supported in chroots");
+			nih_error_raise (EPERM, "user jobs not supported in chroots");
+			job_process_error_abort (fds[1], JOB_PROCESS_ERROR_CHROOT, 0);
 		}
 
 		pw = getpwuid (uid);
 
-		if (!pw)
-			nih_return_system_error (-1);
+		if (!pw) {
+			nih_error_raise_system ();
+			job_process_error_abort (fds[1], JOB_PROCESS_ERROR_GETPWUID, 0);
+		}
 
 		nih_assert (pw->pw_uid == uid);
 
 		if (! pw->pw_dir) {
-			nih_local char *message = NIH_MUST (nih_sprintf (NULL,
-						"no home directory for user with uid %d",
-						uid));
-
-			nih_return_error (-1, ENOENT, message);
+			nih_error_raise_printf (ENOENT,
+					"no home directory for user with uid %d", uid);
+			job_process_error_abort (fds[1], JOB_PROCESS_ERROR_GETPWUID, 0);
 
 		}
 
@@ -682,8 +691,10 @@ job_process_spawn (Job          *job,
 		 */
 		user_dir = nih_strdup (NULL, pw->pw_dir);
 
-		if (! user_dir)
-			nih_return_no_memory_error (-1);
+		if (! user_dir) {
+			nih_error_raise_no_memory ();
+			job_process_error_abort (fds[1], JOB_PROCESS_ERROR_ALLOC, 0);
+		}
 
 		/* Ensure the file associated with fd 9
 		 * (/proc/self/fd/9) is owned by the user we're about to
@@ -1126,6 +1137,11 @@ job_process_error_read (int fd)
 				  err, _("unable to getgrnam: %s"),
 				  strerror (err->errnum)));
 		break;
+	case JOB_PROCESS_ERROR_GETPWUID:
+		err->error.message = NIH_MUST (nih_sprintf (
+				  err, _("unable to getpwuid: %s"),
+				  strerror (err->errnum)));
+		break;
 	case JOB_PROCESS_ERROR_BAD_SETUID:
 		err->error.message = NIH_MUST (nih_sprintf (
 				  err, _("unable to find setuid user")));
@@ -1151,7 +1167,7 @@ job_process_error_read (int fd)
 		break;
 	case JOB_PROCESS_ERROR_OPENPT_MASTER:
 		err->error.message = NIH_MUST (nih_sprintf (
-				  err, _("unable to open pt master: %s"),
+				  err, _("unable to open pty master: %s"),
 				  strerror (err->errnum)));
 		break;
 	case JOB_PROCESS_ERROR_UNLOCKPT:
@@ -1159,6 +1175,11 @@ job_process_error_read (int fd)
 				  err, _("unable to unlockpt: %s"),
 				  strerror (err->errnum)));
 		break;
+	case JOB_PROCESS_ERROR_GRANTPT:
+		err->error.message = NIH_MUST (nih_sprintf (
+				  err, _("unable to granpt: %s"),
+				  strerror (err->errnum)));
+		break;
 	case JOB_PROCESS_ERROR_PTSNAME:
 		err->error.message = NIH_MUST (nih_sprintf (
 				  err, _("unable to get ptsname: %s"),
@@ -1166,7 +1187,17 @@ job_process_error_read (int fd)
 		break;
 	case JOB_PROCESS_ERROR_OPENPT_SLAVE:
 		err->error.message = NIH_MUST (nih_sprintf (
-				  err, _("unable to open pt slave: %s"),
+				  err, _("unable to open pty slave: %s"),
+				  strerror (err->errnum)));
+		break;
+	case JOB_PROCESS_ERROR_SIGNAL:
+		err->error.message = NIH_MUST (nih_sprintf (
+				  err, _("unable to modify signal handler: %s"),
+				  strerror (err->errnum)));
+		break;
+	case JOB_PROCESS_ERROR_ALLOC:
+		err->error.message = NIH_MUST (nih_sprintf (
+				  err, _("unable to allocate memory: %s"),
 				  strerror (err->errnum)));
 		break;
 	default:
@@ -1195,7 +1226,7 @@ job_process_kill (Job         *job,
 	nih_assert (job != NULL);
 	nih_assert (job->pid[process] > 0);
 	nih_assert (job->kill_timer == NULL);
-	nih_assert (job->kill_process = -1);
+	nih_assert (job->kill_process == PROCESS_INVALID);
 
 	nih_info (_("Sending %s signal to %s %s process (%d)"),
 		  nih_signal_to_name (job->class->kill_signal),
@@ -1215,12 +1246,49 @@ job_process_kill (Job         *job,
 		return;
 	}
 
+	job_process_set_kill_timer (job, process, job->class->kill_timeout);
+}
+
+/**
+ * job_process_set_kill_timer:
+ * @job: job to set kill timer for,
+ * @process: process to be killed,
+ * @timeout: timeout to apply for timer.
+ *
+ * Set kill timer for specified @job @process with timeout @timeout.
+ **/
+void
+job_process_set_kill_timer (Job          *job,
+		  	    ProcessType   process,
+			    time_t        timeout)
+{
+	nih_assert (job);
+	nih_assert (timeout);
+
 	job->kill_process = process;
 	job->kill_timer = NIH_MUST (nih_timer_add_timeout (
-			  job, job->class->kill_timeout,
+			  job, timeout,
 			  (NihTimerCb)job_process_kill_timer, job));
 }
 
+/**
+ * job_process_adj_kill_timer:
+ *
+ * @job: job whose kill timer is to be modified,
+ * @due: new due time to set for job kill timer.
+ *
+ * Adjust due time for @job's kill timer to @due.
+ **/
+void
+job_process_adj_kill_timer (Job *job, time_t due)
+{
+	nih_assert (job);
+	nih_assert (job->kill_timer);
+	nih_assert (due);
+
+	job->kill_timer->due = due;
+}
+
 /**
  * job_process_kill_timer:
  * @job: job to kill process of,
@@ -1239,13 +1307,13 @@ job_process_kill_timer (Job      *job,
 	nih_assert (job != NULL);
 	nih_assert (timer != NULL);
 	nih_assert (job->kill_timer == timer);
-	nih_assert (job->kill_process != (ProcessType)-1);
+	nih_assert (job->kill_process != PROCESS_INVALID);
 
 	process = job->kill_process;
 	nih_assert (job->pid[process] > 0);
 
 	job->kill_timer = NULL;
-	job->kill_process = -1;
+	job->kill_process = PROCESS_INVALID;
 
 	nih_info (_("Sending %s signal to %s %s process (%d)"),
 		  "KILL",
@@ -1516,7 +1584,7 @@ job_process_terminated (Job         *job,
 						  job_name (job));
 
 					failed = FALSE;
-					job_failed (job, -1, 0);
+					job_failed (job, PROCESS_INVALID, 0);
 				} else {
 					nih_warn (_("%s %s process ended, respawning"),
 						  job_name (job),
@@ -1600,7 +1668,7 @@ job_process_terminated (Job         *job,
 	if (job->kill_timer) {
 		nih_unref (job->kill_timer, job);
 		job->kill_timer = NULL;
-		job->kill_process = -1;
+		job->kill_process = PROCESS_INVALID;
 	}
 
 	if (job->class->console == CONSOLE_LOG && job->log[process]) {
@@ -2082,6 +2150,18 @@ job_process_log_path (Job *job, int user_job)
 	if (! dir)
 		nih_return_no_memory_error (NULL);
 
+	/* If the job is running inside a chroot, it must be logged to a
+	 * file within the chroot.
+	 */
+	if (job->class->session && job->class->session->chroot) {
+		char *tmp = dir;
+
+		dir = nih_sprintf (NULL, "%s%s",
+				job->class->session->chroot,
+				tmp);
+		nih_free (tmp);
+	}
+
 	class_name = nih_strdup (NULL, class->name);
 
 	if (! class_name)
@@ -2162,8 +2242,10 @@ job_process_remap_fd (int *fd, int reserved_fd, int error_fd)
 		return;
 
 	new = dup (*fd);
-	if (new < 0)
+	if (new < 0) {
+		nih_error_raise_system ();
 		job_process_error_abort (error_fd, JOB_PROCESS_ERROR_DUP, 0);
+	}
 	close (*fd);
 	*fd = new;
 }
