@@ -48,8 +48,13 @@ static int log_flushed = 0;
 /**
  * log_unflushed_files:
  *
- * List of known sources of configuration; each item is an
- * NihListEntry.
+ * List of NihListEntry objects containing Log objects which are no
+ * longer associated with Job processes.
+ *
+ * All the entries in the list contain unflushed Log data.
+ *
+ * Used to capture job process output early in the boot process for
+ * jobs that end before the log partition is mounted and writeable.
  **/
 NihList *log_unflushed_files = NULL;
 
@@ -393,7 +398,7 @@ log_io_error_handler (Log *log, NihIo *io)
  *
  * Opens log file associated with @log if not already open.
  *
- * Returns 0 on success, -1 on failure.
+ * Returns: 0 on success, -1 on failure.
  **/
 static int
 log_file_open (Log *log)
@@ -505,7 +510,7 @@ log_file_open (Log *log)
  *   a corrupted log file should space later become
  *   available.
  *
- * Returns 0 on success, -1 on failure.
+ * Returns: 0 on success, -1 on failure.
  **/
 static int
 log_file_write (Log *log, const char *buf, size_t len)
@@ -638,6 +643,8 @@ log_read_watch (Log *log)
 		if (nih_io_buffer_resize (io->recv_buf, LOG_READ_SIZE) < 0)
 			break;
 
+		errno = 0;
+
 		/* Append to buffer */
 		len = read (io->watch->fd,
 				io->recv_buf->buf + io->recv_buf->len,
@@ -680,22 +687,23 @@ log_read_watch (Log *log)
 		 * causes the loop to be exited.
 		 */
 		if (len <= 0) {
+			/* Job process has ended and we've drained all the data the job
+			 * produced, so remote end must have closed.
+			 *
+			 * This cannot be handled entirely by log_io_error_handler()
+			 * since the job may produce some output prior to disks being
+			 * writeable, then end without producing further output.
+			 * In this scenario the error handler is never called.
+			 *
+			 */
+			if (saved && saved != EAGAIN && saved != EWOULDBLOCK)
+				log->remote_closed = 1;
+
 			close (log->fd);
 			log->fd = -1;
 			break;
 		}
 	}
-
-	/* Job process has ended and we've drained all the data the job
-	 * produced, so remote end must have closed.
-	 *
-	 * This cannot be handled entirely by log_io_error_handler()
-	 * since the job may produce some output prior to disks being
-	 * writeable, then end without producing further output.
-	 * In this scenario the error handler is never called.
-	 *
-	 */
-	log->remote_closed = 1;
 }
 
 /**
@@ -810,3 +818,193 @@ log_clear_unflushed (void)
 
 	return 0;
 }
+
+/**
+ * log_serialise:
+ * @log: log to serialise.
+ *
+ * Convert @log into a JSON representation for serialisation.
+ * Caller must free returned value using json_object_put().
+ *
+ * Returns: JSON-serialised Log object, or NULL on error.
+ **/
+json_object *
+log_serialise (Log *log)
+{
+	json_object     *json;
+	nih_local char  *unflushed_hex = NULL;
+
+	json = json_object_new_object ();
+	if (! json)
+		return NULL;
+
+	if (! log || (! log->io && log->unflushed && ! log->unflushed->len)) {
+		/* Create a "placeholder" log object for non-existent
+		 * log objects and for those that are no longer usable.
+		 */
+		if (! state_set_json_string_var (json, "path", NULL))
+			goto error;
+		return json;
+	}
+
+	/* Attempt to flush any cached data */
+	if (log->unflushed && log->unflushed->len) {
+		/* Don't check return values since if this fails and
+		 * unflushed data remains, we encode it below.
+		 */
+		if (log->fd < 0)
+			(void)log_file_open (log);
+		if (log->fd != -1)
+			(void)log_file_write (log, NULL, 0);
+	}
+
+	nih_assert (log->io);
+
+	if (! state_set_json_int_var_from_obj (json, log, fd))
+		goto error;
+
+	nih_assert (log->io->watch);
+
+	if (! state_set_json_int_var (json, "io_watch_fd", log->io->watch->fd))
+		goto error;
+
+	if (! state_set_json_string_var_from_obj (json, log, path))
+		goto error;
+
+	/* log->io itself is not encoded */
+
+	if (! state_set_json_int_var_from_obj (json, log, uid))
+		goto error;
+
+	/* Encode unflushed data as hex to ensure any embedded
+	 * nulls are handled.
+	 */
+	if (log->unflushed && log->unflushed->len) {
+		unflushed_hex = state_data_to_hex (NULL,
+				log->unflushed->buf,
+				log->unflushed->len);
+
+		if (! unflushed_hex)
+			goto error;
+
+		if (! state_set_json_string_var (json, "unflushed", unflushed_hex))
+			goto error;
+	}
+
+	if (! state_set_json_int_var_from_obj (json, log, detached))
+		goto error;
+
+	if (! state_set_json_int_var_from_obj (json, log, remote_closed))
+		goto error;
+
+	if (! state_set_json_int_var_from_obj (json, log, open_errno))
+		goto error;
+
+	return json;
+
+error:
+	json_object_put (json);
+	return NULL;
+}
+
+/**
+ * log_deserialise:
+ * @json: JSON-serialised Log object to deserialise.
+ *
+ * Convert @json into a Log object.
+ *
+ * Returns: Log object, or NULL on error.
+ **/
+Log *
+log_deserialise (const void *parent,
+		 json_object *json)
+{
+	Log             *log;
+	nih_local char  *unflushed_hex = NULL;
+	nih_local char  *unflushed = NULL;
+	int              ret;
+	size_t           len;
+	json_object     *json_unflushed;
+	nih_local char  *path = NULL;
+	int              io_watch_fd = -1;
+	uid_t            uid = (uid_t)-1;
+
+	nih_assert (json);
+
+	log_unflushed_init ();
+
+	if (! state_check_json_type (json, object))
+		return NULL;
+
+	if (! state_get_json_string_var (json, "path", NULL, path))
+		return NULL;
+
+	if (! path) {
+		/* placeholder log object */
+		return NULL;
+	}
+
+	if (! state_get_json_int_var (json, "io_watch_fd", io_watch_fd))
+		return NULL;
+
+	nih_assert (io_watch_fd != -1);
+
+	/* re-apply CLOEXEC flag to stop job fd being leaked to children */
+	if (state_toggle_cloexec (io_watch_fd, TRUE) < 0)
+		return NULL;
+
+	if (! state_get_json_int_var (json, "uid", uid))
+		return NULL;
+
+	log = log_new (parent, path, io_watch_fd, uid);
+	if (! log)
+		return NULL;
+
+	if (! state_get_json_int_var_to_obj (json, log, fd))
+		goto error;
+
+	/* Re-apply CLOEXEC flag to stop log file fd being leaked to children.
+	 *
+	 * Note we discard return value since if this fails,
+	 * we would never close the fd.
+	 */
+	if (log->fd != -1)
+		(void)state_toggle_cloexec (log->fd, TRUE);
+
+	log->unflushed = nih_io_buffer_new (log);
+	if (! log->unflushed)
+		goto error;
+
+	json_unflushed = json_object_object_get (json, "unflushed");
+	if (json_unflushed) {
+		if (! state_get_json_string_var_strict (json, "unflushed", NULL, unflushed_hex))
+			goto error;
+
+		ret = state_hex_to_data (NULL,
+				unflushed_hex,
+				strlen (unflushed_hex),
+				&unflushed,
+				&len);
+
+		if (ret < 0)
+			goto error;
+
+		if (nih_io_buffer_push (log->unflushed, unflushed, len) < 0)
+			goto error;
+	}
+
+	if (! state_get_json_int_var_to_obj (json, log, detached))
+		goto error;
+
+	if (! state_get_json_int_var_to_obj (json, log, remote_closed))
+		goto error;
+
+	if (! state_get_json_int_var_to_obj (json, log, open_errno))
+		goto error;
+
+	return log;
+
+error:
+	nih_free (log);
+	return NULL;
+}
